@@ -22,7 +22,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -108,19 +108,17 @@ explode_bitmap_into_buf (PnmLoaderContext *context)
 	guchar *from, *to, data;
 	gint bit;
 	guchar *dptr;
-	gint wid, x, y;
+	gint wid, x;
 	
 	g_return_if_fail (context != NULL);
 	g_return_if_fail (context->dptr != NULL);
 	
 	/* I'm no clever bit-hacker so I'm sure this can be optimized */
 	dptr = context->dptr;
-	y    = context->output_row;
 	wid  = context->width;
 	
 	from = dptr + ((wid - 1) / 8);
 	to   = dptr + (wid - 1) * 3;
-/*	bit  = 7 - (((y+1)*wid-1) % 8); */
 	bit  = 7 - ((wid-1) % 8);
 	
 	/* get first byte and align properly */
@@ -135,7 +133,7 @@ explode_bitmap_into_buf (PnmLoaderContext *context)
 		to -= 3;
 		bit++;
 		
-		if (bit > 7) {
+		if (bit > 7 && x > 0) {
 			from--;
 			data = from[0];
 			bit = 0;
@@ -244,12 +242,12 @@ pnm_read_next_value (PnmIOBuffer *inbuf, gint max_length, guint *value, GError *
 	    return PNM_SUSPEND;
 	
 	/* get the value */
-	result = strtol (buf, &endptr, 10);
+	result = strtol ((gchar *)buf, &endptr, 10);
 	if (*endptr != '\0' || result < 0 || result > G_MAXUINT) {
-		g_set_error (error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("PNM loader expected to find an integer, but didn't"));
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("PNM loader expected to find an integer, but didn't"));
 		return PNM_FATAL_ERR;
 	}
 	*value = result;
@@ -278,10 +276,10 @@ pnm_read_header (PnmLoaderContext *context)
 			return PNM_SUSPEND;
 		
 		if (*inbuf->byte != 'P') {
-			g_set_error (context->error,
-				     GDK_PIXBUF_ERROR,
-				     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-				     _("PNM file has an incorrect initial byte"));
+			g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("PNM file has an incorrect initial byte"));
 			return PNM_FATAL_ERR;
 		}
 		
@@ -308,10 +306,10 @@ pnm_read_header (PnmLoaderContext *context)
 			context->type = PNM_FORMAT_PPM_RAW;
 			break;
 		default:
-			g_set_error (context->error,
-				     GDK_PIXBUF_ERROR,
-				     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-				     _("PNM file is not in a recognized PNM subformat"));
+			g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("PNM file is not in a recognized PNM subformat"));
 			return PNM_FATAL_ERR;
 		}
 		
@@ -333,10 +331,10 @@ pnm_read_header (PnmLoaderContext *context)
 			return retval;
 		
 		if (!width) {
-			g_set_error (context->error,
-				     GDK_PIXBUF_ERROR,
-				     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-				     _("PNM file has an image width of 0"));
+			g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("PNM file has an image width of 0"));
 			return PNM_FATAL_ERR;
 		}
 		
@@ -354,10 +352,10 @@ pnm_read_header (PnmLoaderContext *context)
 			return retval;
 		
 		if (!height) {
-			g_set_error (context->error,
-				     GDK_PIXBUF_ERROR,
-				     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-				     _("PNM file has an image height of 0"));
+			g_set_error_literal (context->error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("PNM file has an image height of 0"));
 			return PNM_FATAL_ERR;
 		}
 		
@@ -377,18 +375,18 @@ pnm_read_header (PnmLoaderContext *context)
 				return retval;
 			
 			if (context->maxval == 0) {
-				g_set_error (context->error,
-					     GDK_PIXBUF_ERROR,
-					     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
- 					     _("Maximum color value in PNM file is 0"));
+				g_set_error_literal (context->error,
+                                                     GDK_PIXBUF_ERROR,
+                                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                                     _("Maximum color value in PNM file is 0"));
 				return PNM_FATAL_ERR;
 			}
 
 			if (context->maxval > 65535) {
-				g_set_error (context->error,
-					     GDK_PIXBUF_ERROR,
-					     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
- 					     _("Maximum color value in PNM file is too large"));
+				g_set_error_literal (context->error,
+                                                     GDK_PIXBUF_ERROR,
+                                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                                     _("Maximum color value in PNM file is too large"));
 				return PNM_FATAL_ERR;
 			}
 
@@ -425,10 +423,10 @@ pnm_read_raw_scanline (PnmLoaderContext *context)
 		numpix = inbuf->nbytes / 3;
 		break;
 	default:
-		g_set_error (context->error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("Raw PNM image type is invalid"));
+		g_set_error_literal (context->error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("Raw PNM image type is invalid"));
 		return PNM_FATAL_ERR;
 	}
 	if(context->maxval>255) 
@@ -455,10 +453,10 @@ pnm_read_raw_scanline (PnmLoaderContext *context)
 		offset = context->output_col * 3;
 		break;
 	default:
-		g_set_error (context->error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("Raw PNM image type is invalid"));
+		g_set_error_literal (context->error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("Raw PNM image type is invalid"));
 		return PNM_FATAL_ERR;
 	}
 	if(context->maxval>255) 
@@ -500,10 +498,10 @@ pnm_read_raw_scanline (PnmLoaderContext *context)
 		}
 		break;
 	default:
-		g_set_error (context->error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("Raw PNM image type is invalid"));
+		g_set_error_literal (context->error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("Raw PNM image type is invalid"));
 		return PNM_FATAL_ERR;
 	}
 	
@@ -650,10 +648,10 @@ pnm_read_scanline (PnmLoaderContext *context)
 			return retval;
 		break;
 	default:
-		g_set_error (context->error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
-			     _("PNM image loader does not support this PNM subformat"));
+		g_set_error_literal (context->error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
+                                     _("PNM image loader does not support this PNM subformat"));
 
 		return PNM_FATAL_ERR;
 	}
@@ -705,10 +703,10 @@ gdk_pixbuf__pnm_image_load (FILE *f, GError **error)
 			/* we ran out of data? */
 			if (context.pixbuf)
 				g_object_unref (context.pixbuf);
-			g_set_error (error,
-				     GDK_PIXBUF_ERROR,
-				     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-				     _("Premature end-of-file encountered"));
+			g_set_error_literal (error,
+                                             GDK_PIXBUF_ERROR,
+                                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                             _("Premature end-of-file encountered"));
 			return NULL;
 		}
 		
@@ -737,10 +735,10 @@ gdk_pixbuf__pnm_image_load (FILE *f, GError **error)
 				/* raw formats require exactly one whitespace */
 				if (!g_ascii_isspace(*(inbuf->byte))) 
 					{
-						g_set_error (error,
-							     GDK_PIXBUF_ERROR,
-							     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-							     _("Raw PNM formats require exactly one whitespace before sample data"));
+						g_set_error_literal (error,
+                                                                     GDK_PIXBUF_ERROR,
+                                                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                                                     _("Raw PNM formats require exactly one whitespace before sample data"));
 						return NULL;
 					}
 				inbuf->nbytes--;
@@ -764,10 +762,10 @@ gdk_pixbuf__pnm_image_load (FILE *f, GError **error)
 			
 			if (!context.pixbuf) {
 				/* Failed to allocate memory */
-				g_set_error (error,
-					     GDK_PIXBUF_ERROR,
-					     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-					     _("Cannot allocate memory for loading PNM image"));
+				g_set_error_literal (error,
+                                                     GDK_PIXBUF_ERROR,
+                                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                                     _("Cannot allocate memory for loading PNM image"));
 				return NULL;
 			}
 
@@ -815,9 +813,9 @@ gdk_pixbuf__pnm_image_begin_load (GdkPixbufModuleSizeFunc size_func,
 	
 	context = g_try_malloc (sizeof (PnmLoaderContext));
 	if (!context) {
-		g_set_error(error, GDK_PIXBUF_ERROR, 
-			    GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-			    _("Insufficient memory to load PNM context struct"));
+		g_set_error_literal (error, GDK_PIXBUF_ERROR, 
+                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                     _("Insufficient memory to load PNM context struct"));
 		return NULL;
 	}
 	memset (context, 0, sizeof (PnmLoaderContext));
@@ -865,10 +863,10 @@ gdk_pixbuf__pnm_image_stop_load (gpointer data,
 	   pnm allows to put multiple images in a file
 	*/
 	if (context->inbuf.nbytes > 0) {
-		g_set_error (error,
-			     GDK_PIXBUF_ERROR,
-			     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-			     _("Unexpected end of PNM image data"));
+		g_set_error_literal (error,
+                                     GDK_PIXBUF_ERROR,
+                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                     _("Unexpected end of PNM image data"));
 		retval = FALSE;
 	}
 #endif
@@ -892,8 +890,6 @@ gdk_pixbuf__pnm_image_load_increment (gpointer data,
 {
 	PnmLoaderContext *context = (PnmLoaderContext *)data;
 	PnmIOBuffer *inbuf;
-	guchar *old_byte;
-	guint old_nbytes;
 	const guchar *bufhd;
 	guint num_left, spinguard;
 	gint retval;
@@ -905,8 +901,6 @@ gdk_pixbuf__pnm_image_load_increment (gpointer data,
 	
 	bufhd = buf;
 	inbuf = &context->inbuf;
-	old_nbytes = inbuf->nbytes;
-	old_byte  = inbuf->byte;
 	
 	num_left = size;
 	spinguard = 0;
@@ -968,10 +962,10 @@ gdk_pixbuf__pnm_image_load_increment (gpointer data,
 				/* raw formats require exactly one whitespace */
 				if (!g_ascii_isspace(*(inbuf->byte)))
 					{
-						g_set_error (error,
-							     GDK_PIXBUF_ERROR,
-							     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-							     _("Raw PNM formats require exactly one whitespace before sample data"));
+						g_set_error_literal (error,
+                                                                     GDK_PIXBUF_ERROR,
+                                                                     GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
+                                                                     _("Raw PNM formats require exactly one whitespace before sample data"));
 						return FALSE;
 					}
 				inbuf->nbytes--;
@@ -997,10 +991,10 @@ gdk_pixbuf__pnm_image_load_increment (gpointer data,
 							  context->height);
 			
 			if (context->pixbuf == NULL) {
-				g_set_error (error,
-					     GDK_PIXBUF_ERROR,
-					     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-					     _("Insufficient memory to load PNM file"));
+				g_set_error_literal (error,
+                                                     GDK_PIXBUF_ERROR,
+                                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+                                                     _("Insufficient memory to load PNM file"));
 				return FALSE;
 			}
 			
@@ -1042,8 +1036,13 @@ gdk_pixbuf__pnm_image_load_increment (gpointer data,
 	return TRUE;
 }
 
-void
-MODULE_ENTRY (pnm, fill_vtable) (GdkPixbufModule *module)
+#ifndef INCLUDE_pnm
+#define MODULE_ENTRY(function) G_MODULE_EXPORT void function
+#else
+#define MODULE_ENTRY(function) void _gdk_pixbuf__pnm_ ## function
+#endif
+
+MODULE_ENTRY (fill_vtable) (GdkPixbufModule *module)
 {
 	module->load = gdk_pixbuf__pnm_image_load;
 	module->begin_load = gdk_pixbuf__pnm_image_begin_load;
@@ -1051,8 +1050,7 @@ MODULE_ENTRY (pnm, fill_vtable) (GdkPixbufModule *module)
 	module->load_increment = gdk_pixbuf__pnm_image_load_increment;
 }
 
-void
-MODULE_ENTRY (pnm, fill_info) (GdkPixbufFormat *info)
+MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 {
 	static GdkPixbufModulePattern signature[] = {
 		{ "P1", NULL, 100 },
