@@ -25,6 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include "config.h"
 #include <stdlib.h>
 
 #include "gdkvisual.h"
@@ -60,7 +61,7 @@ gdk_visual_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (GdkVisualClass),
         (GBaseInitFunc) NULL,
@@ -96,20 +97,22 @@ _gdk_visual_init (void)
   HBITMAP hbm;
 
   const gint rastercaps = GetDeviceCaps (_gdk_display_hdc, RASTERCAPS);
+  const int numcolors = GetDeviceCaps (_gdk_display_hdc, NUMCOLORS);
   gint bitspixel = GetDeviceCaps (_gdk_display_hdc, BITSPIXEL);
   gint map_entries = 0;
 
   system_visual = g_object_new (GDK_TYPE_VISUAL, NULL);
 
+  GDK_NOTE (COLORMAP, g_print ("BITSPIXEL=%d NUMCOLORS=%d\n",
+			       bitspixel, numcolors));
+
   if (rastercaps & RC_PALETTE)
     {
       const int sizepalette = GetDeviceCaps (_gdk_display_hdc, SIZEPALETTE);
-      const int numcolors = GetDeviceCaps (_gdk_display_hdc, NUMCOLORS);
       gchar *max_colors = getenv ("GDK_WIN32_MAX_COLORS");
       system_visual->type = GDK_VISUAL_PSEUDO_COLOR;
 
-      GDK_NOTE (COLORMAP, g_print ("BITSPIXEL=%d NUMCOLORS=%d SIZEPALETTE=%d\n",
-				   bitspixel, numcolors, sizepalette));
+      GDK_NOTE (COLORMAP, g_print ("SIZEPALETTE=%d\n", sizepalette));
       g_assert (sizepalette == 256);
 
       if (max_colors != NULL)
@@ -154,6 +157,12 @@ _gdk_visual_init (void)
       else
 	map_entries = sizepalette;
     }
+  else if (bitspixel == 1 && numcolors == 16)
+    {
+      bitspixel = 4;
+      system_visual->type = GDK_VISUAL_STATIC_COLOR;
+      map_entries = 16;
+    }
   else if (bitspixel == 1)
     {
       system_visual->type = GDK_VISUAL_STATIC_GRAY;
