@@ -70,7 +70,7 @@ _gdk_window_impl_win32_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (GdkWindowImplWin32Class),
         (GBaseInitFunc) NULL,
