@@ -22,17 +22,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
-#include <math.h>
-#include <string.h>
-#include "gtkhsv.h"
-#include "gdk/gdkkeysyms.h"
-#include "gtkbindings.h"
-#include "gtkcontainer.h"
-#include "gtkmarshalers.h"
-#include "gtkintl.h"
-#include "gtkalias.h"
-
 /*
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
@@ -40,6 +29,19 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#include "config.h"
+
+#include <math.h>
+#include <string.h>
+
+#include "gdk/gdkkeysyms.h"
+
+#include "gtkhsv.h"
+#include "gtkbindings.h"
+#include "gtkmarshalers.h"
+#include "gtkintl.h"
+#include "gtkalias.h"
+
 /* Default width/height */
 #define DEFAULT_SIZE 100
 
@@ -114,7 +116,7 @@ static void     gtk_hsv_move           (GtkHSV           *hsv,
 
 static guint hsv_signals[LAST_SIGNAL];
 
-G_DEFINE_TYPE (GtkHSV, gtk_hsv, GTK_TYPE_WIDGET);
+G_DEFINE_TYPE (GtkHSV, gtk_hsv, GTK_TYPE_WIDGET)
 
 /* Class initialization function for the HSV color selector */
 static void
@@ -328,9 +330,8 @@ gtk_hsv_unrealize (GtkWidget *widget)
   
   g_object_unref (priv->gc);
   priv->gc = NULL;
-  
-  if (GTK_WIDGET_CLASS (gtk_hsv_parent_class)->unrealize)
-    GTK_WIDGET_CLASS (gtk_hsv_parent_class)->unrealize (widget);
+
+  GTK_WIDGET_CLASS (gtk_hsv_parent_class)->unrealize (widget);
 }
 
 /* Size_request handler for the HSV color selector */
@@ -537,23 +538,25 @@ compute_triangle (GtkHSV *hsv,
 		  gint   *vy)
 {
   HSVPrivate *priv;
-  gdouble center;
+  gdouble center_x;
+  gdouble center_y;
   gdouble inner, outer;
   gdouble angle;
-  
+
   priv = hsv->priv;
-  
-  center = GTK_WIDGET (hsv)->requisition.width / 2.0;
+
+  center_x = GTK_WIDGET (hsv)->allocation.width / 2.0;
+  center_y = GTK_WIDGET (hsv)->allocation.height / 2.0;
   outer = priv->size / 2.0;
   inner = outer - priv->ring_width;
   angle = priv->h * 2.0 * G_PI;
-  
-  *hx = floor (center + cos (angle) * inner + 0.5);
-  *hy = floor (center - sin (angle) * inner + 0.5);
-  *sx = floor (center + cos (angle + 2.0 * G_PI / 3.0) * inner + 0.5);
-  *sy = floor (center - sin (angle + 2.0 * G_PI / 3.0) * inner + 0.5);
-  *vx = floor (center + cos (angle + 4.0 * G_PI / 3.0) * inner + 0.5);
-  *vy = floor (center - sin (angle + 4.0 * G_PI / 3.0) * inner + 0.5);
+
+  *hx = floor (center_x + cos (angle) * inner + 0.5);
+  *hy = floor (center_y - sin (angle) * inner + 0.5);
+  *sx = floor (center_x + cos (angle + 2.0 * G_PI / 3.0) * inner + 0.5);
+  *sy = floor (center_y - sin (angle + 2.0 * G_PI / 3.0) * inner + 0.5);
+  *vx = floor (center_x + cos (angle + 4.0 * G_PI / 3.0) * inner + 0.5);
+  *vy = floor (center_y - sin (angle + 4.0 * G_PI / 3.0) * inner + 0.5);
 }
 
 /* Computes whether a point is inside the hue ring */
@@ -564,18 +567,21 @@ is_in_ring (GtkHSV *hsv,
 {
   HSVPrivate *priv;
   gdouble dx, dy, dist;
-  gdouble center, inner, outer;
-  
+  gdouble center_x;
+  gdouble center_y;
+  gdouble inner, outer;
+
   priv = hsv->priv;
-  
-  center = priv->size / 2.0;
+
+  center_x = GTK_WIDGET (hsv)->allocation.width / 2.0;
+  center_y = GTK_WIDGET (hsv)->allocation.height / 2.0;
   outer = priv->size / 2.0;
   inner = outer - priv->ring_width;
-  
-  dx = x - center;
-  dy = center - y;
+
+  dx = x - center_x;
+  dy = center_y - y;
   dist = dx * dx + dy * dy;
-  
+
   return (dist >= inner * inner && dist <= outer * outer);
 }
 
@@ -589,18 +595,20 @@ compute_sv (GtkHSV  *hsv,
 {
   int ihx, ihy, isx, isy, ivx, ivy;
   double hx, hy, sx, sy, vx, vy;
-  double center;
-  
+  double center_x;
+  double center_y;
+
   compute_triangle (hsv, &ihx, &ihy, &isx, &isy, &ivx, &ivy);
-  center = GTK_WIDGET (hsv)->requisition.width / 2.0;
-  hx = ihx - center;
-  hy = center - ihy;
-  sx = isx - center;
-  sy = center - isy;
-  vx = ivx - center;
-  vy = center - ivy;
-  x -= center;
-  y = center - y;
+  center_x = GTK_WIDGET (hsv)->allocation.width / 2.0;
+  center_y = GTK_WIDGET (hsv)->allocation.height / 2.0;
+  hx = ihx - center_x;
+  hy = center_y - ihy;
+  sx = isx - center_x;
+  sy = center_y - isy;
+  vx = ivx - center_x;
+  vy = center_y - ivy;
+  x -= center_x;
+  y = center_y - y;
   
   if (vx * (x - sx) + vy * (y - sy) < 0.0)
     {
@@ -688,18 +696,20 @@ compute_v (GtkHSV *hsv,
 	   gdouble x,
 	   gdouble y)
 {
-  double center;
+  double center_x;
+  double center_y;
   double dx, dy;
   double angle;
-  
-  center = GTK_WIDGET (hsv)->requisition.width / 2.0;
-  dx = x - center;
-  dy = center - y;
-  
+
+  center_x = GTK_WIDGET (hsv)->allocation.width / 2.0;
+  center_y = GTK_WIDGET (hsv)->allocation.height / 2.0;
+  dx = x - center_x;
+  dy = center_y - y;
+
   angle = atan2 (dy, dx);
   if (angle < 0.0)
     angle += 2.0 * G_PI;
-  
+
   return angle / (2.0 * G_PI);
 }
 
@@ -842,7 +852,6 @@ gtk_hsv_motion (GtkWidget      *widget,
   GtkHSV *hsv;
   HSVPrivate *priv;
   double x, y;
-  gint ix, iy;
   GdkModifierType mods;
   
   hsv = GTK_HSV (widget);
@@ -851,18 +860,11 @@ gtk_hsv_motion (GtkWidget      *widget,
   if (priv->mode == DRAG_NONE)
     return FALSE;
   
-  if (event->is_hint)
-    {
-      gdk_window_get_pointer (priv->window, &ix, &iy, &mods);
-      x = ix;
-      y = iy;
-    }
-  else
-    {
-      x = event->x;
-      y = event->y;
-    }
-  
+  gdk_event_request_motions (event);
+  x = event->x;
+  y = event->y;
+  mods = event->state;
+
   if (priv->mode == DRAG_H)
     {
       gtk_hsv_set_color (hsv, compute_v (hsv, x, y), priv->s, priv->v);
@@ -897,7 +899,8 @@ paint_ring (GtkHSV      *hsv,
   HSVPrivate *priv;
   int xx, yy;
   gdouble dx, dy, dist;
-  gdouble center;
+  gdouble center_x;
+  gdouble center_y;
   gdouble inner, outer;
   guint32 *buf, *p;
   gdouble angle;
@@ -905,6 +908,7 @@ paint_ring (GtkHSV      *hsv,
   gdouble r, g, b;
   cairo_surface_t *source;
   cairo_t *source_cr;
+  gint stride;
   gint focus_width;
   gint focus_pad;
 
@@ -914,25 +918,27 @@ paint_ring (GtkHSV      *hsv,
 			NULL);
   
   priv = hsv->priv;
-  
-  center = widget->requisition.width / 2.0;
-  
+
+  center_x = widget->allocation.width / 2.0;
+  center_y = widget->allocation.height / 2.0;
+
   outer = priv->size / 2.0;
   inner = outer - priv->ring_width;
   
   /* Create an image initialized with the ring colors */
   
-  buf = g_new (guint32, width * height);
+  stride = cairo_format_stride_for_width (CAIRO_FORMAT_RGB24, width);
+  buf = g_new (guint32, height * stride / 4);
   
   for (yy = 0; yy < height; yy++)
     {
       p = buf + yy * width;
       
-      dy = -(yy + y - center);
+      dy = -(yy + y - center_y);
       
       for (xx = 0; xx < width; xx++)
 	{
-	  dx = xx + x - center;
+	  dx = xx + x - center_x;
 	  
 	  dist = dx * dx + dy * dy;
 	  if (dist < ((inner-1) * (inner-1)) || dist > ((outer+1) * (outer+1)))
@@ -958,9 +964,9 @@ paint_ring (GtkHSV      *hsv,
 	}
     }
 
-  source = cairo_image_surface_create_for_data ((char *)buf,
+  source = cairo_image_surface_create_for_data ((unsigned char *)buf,
 						CAIRO_FORMAT_RGB24,
-						width, height, 4 * width);
+						width, height, stride);
 
   /* Now draw the value marker onto the source image, so that it
    * will get properly clipped at the edges of the ring
@@ -977,10 +983,10 @@ paint_ring (GtkHSV      *hsv,
   else
     cairo_set_source_rgb (source_cr, 1., 1., 1.);
 
-  cairo_move_to (source_cr, -x + center, - y + center);
+  cairo_move_to (source_cr, -x + center_x, - y + center_y);
   cairo_line_to (source_cr,
-		 -x + center + cos (priv->h * 2.0 * G_PI) * center,
-		 -y + center - sin (priv->h * 2.0 * G_PI) * center);
+		 -x + center_x + cos (priv->h * 2.0 * G_PI) * priv->size / 2,
+		 -y + center_y - sin (priv->h * 2.0 * G_PI) * priv->size / 2);
   cairo_stroke (source_cr);
   cairo_destroy (source_cr);
 
@@ -994,7 +1000,7 @@ paint_ring (GtkHSV      *hsv,
   cairo_set_line_width (cr, priv->ring_width);
   cairo_new_path (cr);
   cairo_arc (cr,
-	     center, center,
+	     center_x, center_y,
 	     priv->size / 2. - priv->ring_width / 2.,
 	     0, 2 * G_PI);
   cairo_stroke (cr);
@@ -1055,6 +1061,7 @@ paint_triangle (GtkHSV      *hsv,
   cairo_surface_t *source;
   gdouble r, g, b;
   gchar *detail;
+  gint stride;
   
   priv = hsv->priv;
   
@@ -1102,8 +1109,9 @@ paint_triangle (GtkHSV      *hsv,
     }
   
   /* Shade the triangle */
-  
-  buf = g_new (guint32, width * height);
+
+  stride = cairo_format_stride_for_width (CAIRO_FORMAT_RGB24, width);
+  buf = g_new (guint32, height * stride / 4);
   
   for (yy = 0; yy < height; yy++)
     {
@@ -1168,9 +1176,9 @@ paint_triangle (GtkHSV      *hsv,
       }
     }
 
-  source = cairo_image_surface_create_for_data ((char *)buf,
+  source = cairo_image_surface_create_for_data ((unsigned char *)buf,
 						CAIRO_FORMAT_RGB24,
-						width, height, 4 * width);
+						width, height, stride);
   
   /* Draw a triangle with the image as a source */
 
@@ -1356,12 +1364,13 @@ gtk_hsv_focus (GtkWidget       *widget,
 
 /**
  * gtk_hsv_new:
- * @void:
  *
  * Creates a new HSV color selector.
  *
  * Return value: A newly-created HSV color selector.
- **/
+ *
+ * Since: 2.14
+ */
 GtkWidget*
 gtk_hsv_new (void)
 {
@@ -1370,14 +1379,16 @@ gtk_hsv_new (void)
 
 /**
  * gtk_hsv_set_color:
- * @hsv: An HSV color selector.
- * @h: Hue.
- * @s: Saturation.
- * @v: Value.
+ * @hsv: An HSV color selector
+ * @h: Hue
+ * @s: Saturation
+ * @v: Value
  *
- * Sets the current color in an HSV color selector.  Color component values must
- * be in the [0.0, 1.0] range.
- **/
+ * Sets the current color in an HSV color selector.
+ * Color component values must be in the [0.0, 1.0] range.
+ *
+ * Since: 2.14
+ */
 void
 gtk_hsv_set_color (GtkHSV *hsv,
 		   gdouble h,
@@ -1404,16 +1415,21 @@ gtk_hsv_set_color (GtkHSV *hsv,
 
 /**
  * gtk_hsv_get_color:
- * @hsv: An HSV color selector.
- * @h: Return value for the hue.
- * @s: Return value for the saturation.
- * @v: Return value for the value.
+ * @hsv: An HSV color selector
+ * @h: Return value for the hue
+ * @s: Return value for the saturation
+ * @v: Return value for the value
  *
- * Queries the current color in an HSV color selector.  Returned values will be
- * in the [0.0, 1.0] range.
- **/
+ * Queries the current color in an HSV color selector.
+ * Returned values will be in the [0.0, 1.0] range.
+ *
+ * Since: 2.14
+ */
 void
-gtk_hsv_get_color (GtkHSV *hsv, double *h, double *s, double *v)
+gtk_hsv_get_color (GtkHSV *hsv,
+                   double *h,
+                   double *s,
+                   double *v)
 {
   HSVPrivate *priv;
   
@@ -1433,12 +1449,14 @@ gtk_hsv_get_color (GtkHSV *hsv, double *h, double *s, double *v)
 
 /**
  * gtk_hsv_set_metrics:
- * @hsv: An HSV color selector.
- * @size: Diameter for the hue ring.
- * @ring_width: Width of the hue ring.
+ * @hsv: An HSV color selector
+ * @size: Diameter for the hue ring
+ * @ring_width: Width of the hue ring
  *
  * Sets the size and ring width of an HSV color selector.
- **/
+ *
+ * Since: 2.14
+ */
 void
 gtk_hsv_set_metrics (GtkHSV *hsv,
 		     gint    size,
@@ -1467,12 +1485,14 @@ gtk_hsv_set_metrics (GtkHSV *hsv,
 
 /**
  * gtk_hsv_get_metrics:
- * @hsv: An HSV color selector.
- * @size: Return value for the diameter of the hue ring.
- * @ring_width: Return value for the width of the hue ring.
+ * @hsv: An HSV color selector
+ * @size: Return value for the diameter of the hue ring
+ * @ring_width: Return value for the width of the hue ring
  *
  * Queries the size and ring width of an HSV color selector.
- **/
+ *
+ * Since: 2.14
+ */
 void
 gtk_hsv_get_metrics (GtkHSV *hsv,
 		     gint   *size,
@@ -1493,17 +1513,19 @@ gtk_hsv_get_metrics (GtkHSV *hsv,
 
 /**
  * gtk_hsv_is_adjusting:
- * @hsv:
+ * @hsv: A #GtkHSV 
  *
- * An HSV color selector can be said to be adjusting if multiple rapid changes
- * are being made to its value, for example, when the user is adjusting the
- * value with the mouse.  This function queries whether the HSV color selector
- * is being adjusted or not.
+ * An HSV color selector can be said to be adjusting if multiple rapid
+ * changes are being made to its value, for example, when the user is 
+ * adjusting the value with the mouse. This function queries whether 
+ * the HSV color selector is being adjusted or not.
  *
- * Return value: TRUE if clients can ignore changes to the color value, since
- * they may be transitory, or FALSE if they should consider the color value
- * status to be final.
- **/
+ * Return value: %TRUE if clients can ignore changes to the color value,
+ *     since they may be transitory, or %FALSE if they should consider
+ *     the color value status to be final.
+ *
+ * Since: 2.14
+ */
 gboolean
 gtk_hsv_is_adjusting (GtkHSV *hsv)
 {
@@ -1518,16 +1540,19 @@ gtk_hsv_is_adjusting (GtkHSV *hsv)
 
 /**
  * gtk_hsv_to_rgb:
- * @h: Hue.
- * @s: Saturation.
- * @v: Value.
- * @r: Return value for the red component.
- * @g: Return value for the green component.
- * @b: Return value for the blue component.
- * 
- * Converts a color from HSV space to RGB.  Input values must be in the
- * [0.0, 1.0] range; output values will be in the same range.
- **/
+ * @h: Hue
+ * @s: Saturation
+ * @v: Value
+ * @r: Return value for the red component
+ * @g: Return value for the green component
+ * @b: Return value for the blue component
+ *
+ * Converts a color from HSV space to RGB.
+ * Input values must be in the [0.0, 1.0] range; 
+ * output values will be in the same range.
+ *
+ * Since: 2.14
+ */
 void
 gtk_hsv_to_rgb (gdouble  h,
 		gdouble  s,
@@ -1553,17 +1578,20 @@ gtk_hsv_to_rgb (gdouble  h,
 }
 
 /**
- * gtk_hsv_to_rgb:
- * @r: Red.
- * @g: Green.
- * @b: Blue.
- * @h: Return value for the hue component.
- * @s: Return value for the saturation component.
- * @v: Return value for the value component.
- * 
- * Converts a color from RGB space to HSV.  Input values must be in the
- * [0.0, 1.0] range; output values will be in the same range.
- **/
+ * gtk_rgb_to_hsv:
+ * @r: Red
+ * @g: Green
+ * @b: Blue
+ * @h: Return value for the hue component
+ * @s: Return value for the saturation component
+ * @v: Return value for the value component
+ *
+ * Converts a color from RGB space to HSV.
+ * Input values must be in the [0.0, 1.0] range;
+ * output values will be in the same range.
+ *
+ * Since: 2.14
+ */
 void
 gtk_rgb_to_hsv (gdouble  r,
 		gdouble  g,
