@@ -28,35 +28,33 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#include "config.h"
 
-#include <config.h>
-#include <glib/gprintf.h>
 #include <stdlib.h>
+
 #include "gdk/gdkkeysyms.h"
+
+#undef GTK_DISABLE_DEPRECATED /* GtkOptionMenu */
+
+#include "gtkinputdialog.h"
 #include "gtkbutton.h"
 #include "gtkentry.h"
 #include "gtkhbox.h"
-#include "gtkhseparator.h"
-#include "gtkinputdialog.h"
 #include "gtklabel.h"
-#include "gtklistitem.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtkmenu.h"
 #include "gtkmenuitem.h"
 #include "gtknotebook.h"
+#include "gtkoptionmenu.h"
 #include "gtkscrolledwindow.h"
 #include "gtkstock.h"
 #include "gtktable.h"
 #include "gtkvbox.h"
 
-#undef GTK_DISABLE_DEPRECATED
-#include "gtkoptionmenu.h"
-#define GTK_DISABLE_DEPRECATED
-
 #include "gtkintl.h"
 #include "gtkalias.h"
 
@@ -119,7 +117,7 @@ static void gtk_input_dialog_fill_keys        (GtkInputDialog      *inputd,
 
 static guint input_dialog_signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (GtkInputDialog, gtk_input_dialog, GTK_TYPE_DIALOG);
+G_DEFINE_TYPE (GtkInputDialog, gtk_input_dialog, GTK_TYPE_DIALOG)
 
 static GtkInputDialogPrivate *
 gtk_input_dialog_get_private (GtkInputDialog *input_dialog)
@@ -156,7 +154,7 @@ gtk_input_dialog_class_init (GtkInputDialogClass *klass)
   klass->disable_device = NULL;
 
   input_dialog_signals[ENABLE_DEVICE] =
-    g_signal_new (I_("enable_device"),
+    g_signal_new (I_("enable-device"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkInputDialogClass, enable_device),
@@ -166,7 +164,7 @@ gtk_input_dialog_class_init (GtkInputDialogClass *klass)
 		  GDK_TYPE_DEVICE);
 
   input_dialog_signals[DISABLE_DEVICE] =
-    g_signal_new (I_("disable_device"),
+    g_signal_new (I_("disable-device"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkInputDialogClass, disable_device),
@@ -485,8 +483,10 @@ gtk_input_dialog_set_axis (GtkWidget *w,
   else
     old_use = GDK_AXIS_IGNORE;
 
-  if (axis == old_axis)
+  if (axis == old_axis) {
+    g_free (new_axes);
     return;
+  }
 
   /* we must always have an x and a y axis */
   if ((axis == -1 && (use == GDK_AXIS_X || use == GDK_AXIS_Y)) ||
@@ -534,6 +534,7 @@ gtk_input_dialog_fill_axes(GtkInputDialog *inputd, GdkDevice *info)
   GtkWidget *option_menu;
   GtkWidget *label;
   GtkWidget *viewport;
+  GtkWidget *old_child;
 
   /* remove all the old items */
   if (inputd->axis_list)
@@ -546,6 +547,9 @@ gtk_input_dialog_fill_axes(GtkInputDialog *inputd, GdkDevice *info)
   gtk_table_set_col_spacings (GTK_TABLE (inputd->axis_list), 12);
   
   viewport = gtk_viewport_new (NULL, NULL);
+  old_child = gtk_bin_get_child (GTK_BIN (inputd->axis_listbox));
+  if (old_child != NULL)
+    gtk_widget_destroy (old_child);
   gtk_container_add (GTK_CONTAINER (inputd->axis_listbox), viewport);
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
   gtk_widget_show (viewport);
@@ -663,7 +667,7 @@ gtk_input_dialog_key_press (GtkWidget *widget,
   gdk_device_set_key (key->inputd->current_device, key->index, 
 		      event->keyval, event->state & 0xFF);
 
-  g_signal_stop_emission_by_name (widget, "key_press_event");
+  g_signal_stop_emission_by_name (widget, "key-press-event");
   
   return TRUE;
 }
@@ -682,6 +686,7 @@ gtk_input_dialog_fill_keys(GtkInputDialog *inputd, GdkDevice *info)
   GtkWidget *button;
   GtkWidget *hbox;
   GtkWidget *viewport;
+  GtkWidget *old_child;
 
   char buffer[32];
   
@@ -697,6 +702,9 @@ gtk_input_dialog_fill_keys(GtkInputDialog *inputd, GdkDevice *info)
   gtk_table_set_col_spacings (GTK_TABLE (inputd->keys_list), 12);
   
   viewport = gtk_viewport_new (NULL, NULL);
+  old_child = gtk_bin_get_child (GTK_BIN (inputd->keys_listbox));
+  if (old_child != NULL)
+    gtk_widget_destroy (old_child);
   gtk_container_add (GTK_CONTAINER (inputd->keys_listbox), viewport);
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
   gtk_widget_show (viewport);
@@ -733,7 +741,7 @@ gtk_input_dialog_fill_keys(GtkInputDialog *inputd, GdkDevice *info)
       gtk_box_pack_start (GTK_BOX (hbox), key->entry, TRUE, TRUE, 0);
       gtk_widget_show (key->entry);
 
-      g_signal_connect (key->entry, "key_press_event",
+      g_signal_connect (key->entry, "key-press-event",
 			G_CALLBACK (gtk_input_dialog_key_press), key);
       g_signal_connect (key->entry, "destroy",
 			G_CALLBACK (gtk_input_dialog_destroy_key), key);
