@@ -17,17 +17,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "locale.h"
 #include <string.h>
 #include <stdlib.h>
 
-#include "gtk/gtkintl.h"
-#include "gtk/gtklabel.h"
-#include "gtk/gtksignal.h"
-#include "gtk/gtkwindow.h"
 #include "gtkimcontextxim.h"
 
+#include "gtk/gtkintl.h"
+
 typedef struct _StatusWindow StatusWindow;
 typedef struct _GtkXIMInfo GtkXIMInfo;
 
@@ -86,6 +84,7 @@ struct _GtkXIMInfo
   GtkSettings *settings;
   gulong status_set;
   gulong preedit_set;
+  gulong display_closed_cb;
   XIMStyles *xim_styles;
   GSList *ics;
 
@@ -143,6 +142,10 @@ static void xim_destroy_callback   (XIM      xim,
 				    XPointer call_data);
 
 static XIC       gtk_im_context_xim_get_ic            (GtkIMContextXIM *context_xim);
+static void           xim_info_display_closed (GdkDisplay *display,
+			                       gboolean    is_error,
+			                       GtkXIMInfo *info);
+
 static GObjectClass *parent_class;
 
 GType gtk_type_im_context_xim = 0;
@@ -155,7 +158,7 @@ static GSList *status_windows = NULL;
 void
 gtk_im_context_xim_register_type (GTypeModule *type_module)
 {
-  static const GTypeInfo im_context_xim_info =
+  const GTypeInfo im_context_xim_info =
   {
     sizeof (GtkIMContextXIMClass),
     (GBaseInitFunc) NULL,
@@ -307,6 +310,7 @@ setup_im (GtkXIMInfo *info)
 {
   XIMValuesList *ic_values = NULL;
   XIMCallback im_destroy_callback;
+  GdkDisplay *display;
 
   if (info->im == NULL)
     return;
@@ -376,6 +380,10 @@ setup_im (GtkXIMInfo *info)
 
   status_style_change (info);
   preedit_style_change (info);
+
+  display = gdk_screen_get_display (info->screen);
+  info->display_closed_cb = g_signal_connect (display, "closed",
+	                                      G_CALLBACK (xim_info_display_closed), info);
 }
 
 static void
@@ -394,12 +402,16 @@ xim_info_display_closed (GdkDisplay *display,
     set_ic_client_window (tmp_list->data, NULL);
 
   g_slist_free (ics);
-  
-  g_signal_handler_disconnect (info->settings, info->status_set);
-  g_signal_handler_disconnect (info->settings, info->preedit_set);
-  
-  XFree (info->xim_styles->supported_styles);
-  XFree (info->xim_styles);
+
+  if (info->status_set)
+    g_signal_handler_disconnect (info->settings, info->status_set);
+  if (info->preedit_set)
+    g_signal_handler_disconnect (info->settings, info->preedit_set);
+  if (info->display_closed_cb)
+    g_signal_handler_disconnect (display, info->display_closed_cb);
+
+  if (info->xim_styles)
+    XFree (info->xim_styles);
   g_free (info->locale);
 
   if (info->im)
@@ -455,9 +467,6 @@ xim_info_try_im (GtkXIMInfo *info)
 	  return;
 	}
       setup_im (info);
-
-      g_signal_connect (display, "closed",
-			G_CALLBACK (xim_info_display_closed), info);
     }
 }
 
@@ -471,7 +480,9 @@ xim_destroy_callback (XIM      xim,
   info->im = NULL;
 
   g_signal_handler_disconnect (info->settings, info->status_set);
+  info->status_set = 0;
   g_signal_handler_disconnect (info->settings, info->preedit_set);
+  info->preedit_set = 0;
 
   reinitialize_all_ics (info);
   xim_info_try_im (info);
@@ -520,6 +531,7 @@ get_im (GdkWindow *client_window,
       info->settings = NULL;
       info->preedit_set = 0;
       info->status_set = 0;
+      info->display_closed_cb = 0;
       info->ics = NULL;
       info->reconnecting = FALSE;
       info->im = NULL;
@@ -610,7 +622,7 @@ reinitialize_ic (GtkIMContextXIM *context_xim)
 	{
 	  context_xim->preedit_length = 0;
 	  if (!context_xim->finalizing)
-	    g_signal_emit_by_name (context_xim, "preedit_changed");
+	    g_signal_emit_by_name (context_xim, "preedit-changed");
 	}
     }
   /* 
@@ -834,7 +846,7 @@ gtk_im_context_xim_set_cursor_location (GtkIMContext *context,
     return;
 
   spot.x = area->x;
-  spot.y = area->y;
+  spot.y = area->y + area->height;
 
   preedit_attr = XVaCreateNestedList (0,
 				      XNSpotLocation, &spot,
@@ -918,7 +930,7 @@ gtk_im_context_xim_reset (GtkIMContext *context)
   if (context_xim->preedit_length)
     {
       context_xim->preedit_length = 0;
-      g_signal_emit_by_name (context, "preedit_changed");
+      g_signal_emit_by_name (context, "preedit-changed");
     }
 
   XFree (result);
@@ -1020,7 +1032,7 @@ preedit_start_callback (XIC      xic,
   GtkIMContextXIM *context_xim = GTK_IM_CONTEXT_XIM (context);
   
   if (!context_xim->finalizing)
-    g_signal_emit_by_name (context, "preedit_start");
+    g_signal_emit_by_name (context, "preedit-start");
 
   return -1;			/* No length limit */
 }		     
@@ -1037,11 +1049,11 @@ preedit_done_callback (XIC      xic,
     {
       context_xim->preedit_length = 0;
       if (!context_xim->finalizing)
-	g_signal_emit_by_name (context_xim, "preedit_changed");
+	g_signal_emit_by_name (context_xim, "preedit-changed");
     }
 
   if (!context_xim->finalizing)
-    g_signal_emit_by_name (context, "preedit_end");  
+    g_signal_emit_by_name (context, "preedit-end");
 }		     
 
 static gint
@@ -1166,7 +1178,7 @@ preedit_draw_callback (XIC                           xic,
   g_free (new_text);
 
   if (!context->finalizing)
-    g_signal_emit_by_name (context, "preedit_changed");
+    g_signal_emit_by_name (context, "preedit-changed");
 }
     
 
@@ -1181,7 +1193,7 @@ preedit_caret_callback (XIC                            xic,
     {
       context->preedit_cursor = call_data->position;
       if (!context->finalizing)
-	g_signal_emit_by_name (context, "preedit_changed");
+	g_signal_emit_by_name (context, "preedit-changed");
     }
   else
     {
@@ -1742,7 +1754,7 @@ status_window_get (GtkWidget *toplevel)
   g_signal_connect (toplevel, "destroy",
 		    G_CALLBACK (on_status_toplevel_destroy),
 		    status_window);
-  g_signal_connect (toplevel, "configure_event",
+  g_signal_connect (toplevel, "configure-event",
 		    G_CALLBACK (on_status_toplevel_configure),
 		    status_window);
   g_signal_connect (toplevel, "notify::screen",
@@ -1810,11 +1822,11 @@ status_window_make_window (StatusWindow *status_window)
   gtk_misc_set_padding (GTK_MISC (status_label), 1, 1);
   gtk_widget_show (status_label);
   
-  g_signal_connect (window, "style_set",
+  g_signal_connect (window, "style-set",
 		    G_CALLBACK (on_status_window_style_set), status_label);
   gtk_container_add (GTK_CONTAINER (window), status_label);
   
-  g_signal_connect (window, "expose_event",
+  g_signal_connect (window, "expose-event",
 		    G_CALLBACK (on_status_window_expose_event), NULL);
   
   gtk_window_set_screen (GTK_WINDOW (status_window->window),
@@ -1860,4 +1872,13 @@ gtk_im_context_xim_shutdown (void)
 {
   while (status_windows)
     status_window_free (status_windows->data);
+
+  while (open_ims)
+    {
+      GtkXIMInfo *info = open_ims->data;
+      GdkDisplay *display = gdk_screen_get_display (info->screen);
+
+      xim_info_display_closed (display, FALSE, info);
+      open_ims = g_slist_remove_link (open_ims, open_ims);
+    }
 }
